package com.monkeycoder.monkeynes.base;

import java.io.*;
import java.util.*;

/**
 * This provides completely generic file filtering for Swing based FileChooser dialogs.
 * <br><br> 
 * Originally part of the MonkeyNES project.
 *
 * @author David Stephens
 */
public class GenericFileFilter extends javax.swing.filechooser.FileFilter implements FileFilter
{
  private ArrayList extensions   = new ArrayList();
  private String    description  = "";
  private boolean   showDirs     = false;

  /** 
   * Creates an empty GenericFileFilter with no extensions, no description, and not showing directories.
   */
  public GenericFileFilter()
  {
    // Do nothing...
  }

  /** 
   * Creates a GenericFileFilter with the extensions provided, no description, and not showing directories.
   * @param inExtensions An ArrayList containing the extensions.
   */
  public GenericFileFilter( ArrayList inExtensions )
  {
    extensions  = inExtensions;
  }  

  /** 
   * Creates a GenericFileFilter with the extensions provided, no description, and optionally showing directories.
   * @param inExtensions An ArrayList containing the extensions.
   * @param inShowDirs Whether or not to display directories with this filter.
   */  
  public GenericFileFilter( ArrayList inExtensions, boolean inShowDirs )
  {
    extensions  = inExtensions;
    showDirs    = inShowDirs;
  }

  /** 
   * Creates a GenericFileFilter with the extensions and description provided, and not showing directories.
   * @param inExtensions An ArrayList containing the extensions.
   * @param inDescription The description of this filter in the drop down list.
   */  
  public GenericFileFilter( ArrayList inExtensions, String inDescription )
  {
    extensions  = inExtensions;
    description = inDescription;
  }  

  /** 
   * Creates a GenericFileFilter with the extensions and description provided, and optionally showing directories.
   * @param inExtensions An ArrayList containing the extensions.
   * @param inDescription The description of this filter in the drop down list.
   * @param inShowDirs Whether or not to display directories with this filter.
   */  
  public GenericFileFilter( ArrayList inExtensions, String inDescription, boolean inShowDirs )
  {
    extensions  = inExtensions;
    description = inDescription;
    showDirs    = inShowDirs;
  }    
  
  /** 
   * Creates a GenericFileFilter with the extensions provided, no description, and not showing directories.
   * @param inExtensions A pipe ('|') seperated string containing the extensions.
   */  
  public GenericFileFilter( String inExtensions )
  {
    StringTokenizer st = new StringTokenizer( inExtensions, "|" );
    
    while( st.hasMoreTokens() )
      addExtension( st.nextToken() );
  }
  
  /** 
   * Creates a GenericFileFilter with the extensions provided, no description, and optionally showing directories.
   * @param inExtensions A pipe ('|') seperated string containing the extensions.
   * @param inShowDirs Whether or not to display directories with this filter.
   */
  public GenericFileFilter( String inExtensions, boolean inShowDirs )
  {
    StringTokenizer st = new StringTokenizer( inExtensions, "|" );
    
    while( st.hasMoreTokens() )
      addExtension( st.nextToken() );
    
    setShowDirs( inShowDirs );
  }  

  /** 
   * Creates a GenericFileFilter with the extensions and description provided, and not showing directories.
   * @param inExtensions A pipe ('|') seperated string containing the extensions.
   * @param inDescription The description of this filter in the drop down list.
   */  
  public GenericFileFilter( String inExtensions, String inDescription )
  {
    StringTokenizer st = new StringTokenizer( inExtensions, "|" );
    
    while( st.hasMoreTokens() )
      addExtension( st.nextToken() );
      
    setDescription( inDescription );
  }  

  /** 
   * Creates a GenericFileFilter with the extensions and description provided, and optionally showing directories.
   * @param inExtensions A pipe ('|') seperated string containing the extensions.
   * @param inDescription The description of this filter in the drop down list.
   * @param inShowDirs Whether or not to display directories with this filter.
   */  
  public GenericFileFilter( String inExtensions, String inDescription, boolean inShowDirs )
  {
    StringTokenizer st = new StringTokenizer( inExtensions, "|" );
    
    while( st.hasMoreTokens() )
      addExtension( st.nextToken() );
      
    setDescription( inDescription );
    setShowDirs( inShowDirs );
  }    
  
  /** 
   * Adds an extension to the list of extensions on which this GenericFileFilter filters.
   * @param inExt A string containing the extension to add.
   */  
  public void addExtension( String inExt )
  {
    // Rip off the . and stuff if they exist
    if( inExt.indexOf( "." ) != -1 )
      inExt = inExt.substring( inExt.indexOf( "." ) + 1 );
      
    extensions.add( inExt );
  }

  /** 
   * Sets the description.
   * @param inDesc The description of this filter in the drop down list.
   */  
  public void setDescription( String inDesc )
  {
    description = inDesc;
  }
  
  /** 
   * Gets the description.
   * @return The description that will show up in the drop down list.
   */  
  public String getDescription()
  {
    return description; 
  }

  /** 
   * Sets whether or not directories will be shown.
   * @param inShowDirs Whether or not to display directories with this filter.
   */  
  public void setShowDirs( boolean inShowDirs )
  {
    showDirs = inShowDirs;
  }  

  /** 
   * Returns whether or not the specified file is to be shown.
   * @param inFile The file being tested.
   * @return Whether or not the specified file is to be shown.
   */
  public boolean accept( File inFile ) 
  {
    if( showDirs && inFile.isDirectory() )
      return true;
    
    if( extensions != null )
    {
      for( int i = 0; i < extensions.size(); i++ )
      {
        if( inFile.getName().substring( inFile.getName().length() - ((String)extensions.get(i)).length() ).equalsIgnoreCase( (String)extensions.get(i) ) )
          return true;
      }
    }
    else
      return true;
    
    return false;
  }
}