package com.monkeycoder.monkeynes.base;

import java.util.*;

/**
 * This is a data transport class used as a message.  It describes exactly
 * what memory was accessed and how.
 * <br><br>
 * Originally part of the MonkeyNES project.
 *
 * @author David Stephens
 */
public class MemoryAccessMessage
{
  public static final int MEM_MAIN         = 0;
  public static final int MEM_VIDEO        = 1;

  public static final int TYPE_NONE        = 0;
  public static final int TYPE_READ        = 1;
  public static final int TYPE_WRITE       = 2;
  public static final int TYPE_READ_WORD   = 3;
  public static final int TYPE_WRITE_WORD  = 4;
  public static final int TYPE_READ_RANGE  = 5;
  public static final int TYPE_WRITE_RANGE = 6;

  private static final String[] bankName = { "Main", "Video" };
  private static final String[] typeName = { "None", "Read", "Write", "Read Word", "Write Word", "Read Range", "Write Range" };

  private int memBank        = MEM_MAIN;
  private int type           = TYPE_NONE;
  private int addrStart      = 0;
  private int addrStop       = 0;

  /**
   * Creates a new MemoryAccessMessage based on a single byte access.
   * @param inMemBank Which memory bank was accessed.
   * @param inType Type of access that was performed.
   * @param inAddr Address in memory that was accessed.
   */
  public MemoryAccessMessage( int inMemBank, int inType, int inAddr )
  {
    memBank   = inMemBank;
    type      = inType;
    addrStart = inAddr;
    addrStop  = inAddr;
  }

  /**
   * Creates a new MemoryAccessMessage based on a multiple byte access.
   * The memory range identified is inclusive.
   * @param inMemBank Which memory bank was accessed.
   * @param inType Type of access that was performed.
   * @param inAddrStart The address where access began.
   * @param inAddrStop The address where access ended.
   */
  public MemoryAccessMessage( int inMemBank, int inType, int inAddrStart, int inAddrStop )
  {
    memBank   = inMemBank;
    type      = inType;
    addrStart = inAddrStart;
    addrStop  = inAddrStop;
  }

  /**
   * Returns which memory bank was accessed.
   * @return Which memory bank was accessed.
   */
  public int getMemoryBank()
  {
    return memBank;
  }

  /**
   * Returns the type of memory access that was performed.
   * @return The type of memory access that was performed.
   */
  public int getType()
  {
    return type;
  }

  /**
   * For single byte access, this method returns the address of the accessed byte.
   * @return The address of the accessed byte.
   */
  public int getAddr()
  {
    return addrStart;
  }

  /**
   * For multiple byte access, this method returns the start address of the accessed memory.
   * @return The start address of the accessed memory.
   */
  public int getAddrStart()
  {
    return addrStart;
  }

  /**
   * For multiple byte access, this method returns the stop address of the accessed memory.
   * @return The stop address of the accessed memory.
   */
  public int getAddrStop()
  {
    return addrStop;
  }

  /**
   * Converts the data in this message to a string.
   * @return A string representing this memory access.
   */
  public String toString()
  {
    StringBuffer sb = new StringBuffer();

    sb.append( "MemoryAccessMessage\n" );
    sb.append( "  Memory Bank:   " );
    sb.append( bankName[getMemoryBank()] );
    sb.append( "\n  Access Type:   " );
    sb.append( typeName[getType()] );
    sb.append( "\n  Start Addr:    " );
    sb.append( getAddrStart() );
    sb.append( "\n  Stop Addr:     " );
    sb.append( getAddrStop() );

    return sb.toString();
  }
}